<?php

/**
 * This is a sample defect plugin implementation for TestRail. See the
 * related article here: 
 *
 * http://docs.gurock.com/testrail-integration/defects-plugins-custom
 *
 * Copyright 2011 Gurock Software GmbH. All rights reserved.
 * http://www.gurock.com - contact@gurock.com
 */

class Bugs_defect_plugin extends Defect_plugin
{
	public function get_meta()
	{
		return array(
			'author' => 'Example Inc.',
			'version' => '1.0',
			'description' => 'Bugs defect plugin for TestRail',
			'can_push' => true,
			'can_lookup' => true,
			'default_config' => 
				"; Please configure your Bugs connection below\n" .
				"[connection]\n" .
				"address=http://<your-server>/\n" .
				"user=testrail\n" .
				"password=secret"
		);
	}


	public function validate_config($config)
	{
		$ini = ini::parse($config);
		
		// Check if the [connection] section exists
		if (!isset($ini['connection']))
		{
			throw new ValidationException('Missing [connection] group');
		}
		
		// Check if required values exist
		$keys = array('address', 'user', 'password');
		foreach ($keys as $key)
		{
			if (!isset($ini['connection'][$key]) ||
				!$ini['connection'][$key])
			{
				throw new ValidationException(
					"Missing configuration for key '$key'"
				);
			}
		}
		
		$address = $ini['connection']['address'];
		
		// Check whether the address is a valid url (syntax only)
		if (!check::url($address))
		{
			throw new ValidationException('Address is not a valid url');
		}
	}

	public function configure($config)
	{
		$ini = ini::parse($config);
		$this->_address = str::slash($ini['connection']['address']);
		$this->_user = $ini['connection']['user'];
		$this->_password = $ini['connection']['password'];	
	}

	public function lookup($defect_id)
	{
		// Get the defect information via the Bugs API
		$defect = $this->_bugs_get_defect($defect_id);

		// Build the status_id based on the status property
		if ($defect['status'] == 'Open')
		{
			$status_id = GI_DEFECTS_STATUS_OPEN;
		}
		elseif ($defect['status'] == 'Resolved')
		{
			$status_id = GI_DEFECTS_STATUS_RESOLVED;
		}
		else
		{
			$status_id = GI_DEFECTS_STATUS_CLOSED;
		}
		
		// Build the bug URL and project link
		$bug_url = str::format('{0}?bug={1}', 
			$this->_address,
			$defect['id']);
		$project_link = str::format(
			'<a target="_blank" href="{0}project={1}">{2}</a>',
			a($this->_address),
			a($defect['project_id']),
			h($defect['project']));
		
		// Build the description
		$description = str::format(
			'<div class="monospace">{0}</div>',
			nl2br(
				html::link_urls(
					h($defect['description'])
				)
			)
		);	
		
		// Return the defect details
		return array(
			'id' => $defect['id'],
			'title' => $defect['title'],
			'status_id' => $status_id,
			'status' => $defect['status'],
			'url' => $bug_url,
			'description' => $description,
			'attributes' => array(
				'Type' => h($defect['type']),
				'Status' => h($defect['status']),
				'Project' => $project_link
			)
		);
	}

	public function prepare_push($context)
	{
		// Return a form with the following fields/properties
		return array(
			'fields' => array(
				'summary' => array(
					'type' => 'string',
					'label' => 'Summary',
					'required' => true,
					'size' => 'full'
				),
				'type' => array(
					'type' => 'dropdown',
					'label' => 'Issue Type',
					'required' => true,
					'remember' => true,
					'size' => 'compact'
				),
				'project' => array(
					'type' => 'dropdown',
					'label' => 'Project',
					'required' => true,
					'remember' => true,
					'cascading' => true,
					'size' => 'compact'
				),
				'component' => array(
					'type' => 'dropdown',
					'label' => 'Component',
					'required' => true,
					'remember' => true,
					'depends_on' => 'project',
					'size' => 'compact'
				),
				'description' => array(
					'type' => 'text',
					'label' => 'Description'
				)
			)
		);
	}

	private function _get_summary_default($context)
	{
		$test = current($context['tests']);
		$summary = 'Failed test: ' . $test->case->title;
		
		if ($context['test_count'] > 1)
		{
			$summary .= ' (+others)';
		}
		
		return $summary;
	}

	private function _get_description_default($context)
	{
		if ($context['test_change']->comment)
		{
			$comment = $context['test_change']->comment;
			$comment .= "\n";
		}
		else 
		{
			$comment = '';
		}
		
		$tests = $context['tests'];
		foreach ($tests as $test)
		{
			$comment .= "\nTest: ";
			$comment .= $test->case->title;
			$comment .= "\n";
			$comment .= $test->url;
		}
		
		return $comment;
	}

	public function prepare_field($context, $input, $field)
	{
		$data = array();
		
		// Take the preferences of the user into account, but only
		// for the initial form rendering (not for cascading loads).
		if ($context['event'] == 'prepare')
		{
			$prefs = arr::get($context, 'preferences');
		}
		else
		{
			$prefs = null;
		}
		
		// And then build the options and default values for the
		// form fields and return them.
		switch ($field)
		{
			case 'summary':
				$data['default'] = $this->_get_summary_default($context);
				break;
				
			case 'description':
				$data['default'] = $this->_get_description_default($context);
				break;
				
			case 'type':
				$data['options'] = $this->_bugs_get_types();
				
				// Select the stored preference or the first item in
				// the list otherwise.
				$default = arr::get($prefs, 'type');
				if ($default)
				{
					$data['default'] = $default;
				}
				else
				{
					if ($data['options'])
					{
						$data['default'] = key($data['options']);
					}
				}				
				break;

			case 'project':
				$data['default'] = arr::get($prefs, 'project');
				$data['options'] = $this->_bugs_get_projects();
				break;

			case 'component':
				if (isset($input['project']))
				{
					$data['default'] = arr::get($prefs, 'component');
					$data['options'] = $this->_bugs_get_components(
						$input['project']);
				}
				break;
		}
		
		return $data;
	}

	public function validate_push($context, $input)
	{
		if (isset($input['summary']))
		{
			if (str::len($input['summary']) > 100)
			{
				throw new ValidationException(
					'Field Summary cannot exceed 100 characters.');
			}
		}
	}

	public function push($context, $input)
	{
		// Push the defect and return the new defect ID
		$defect_id = $this->_bugs_push_defect(
			$input['summary'],
			$input['description'],
			$input['project'],
			$input['type'],
			$input['component']
		);
		return $defect_id;
	}

	// "Bugs" API dummy methods begin here
	private function _bugs_push_defect($summary, $comment,
		$project, $type, $component)
	{
		return rand(1000, 2000);
	}

	private function _bugs_get_defect($id)
	{
		return array(
			'id' => $id,
			'title' => 'CSV Import: Could not detect file encoding',
			'status' => 'Open',
			'description' => 'Importing our standard Unicode CSV file (unitest1.csv) failed with the error message `Could not detect file encoding`.',
			'type' => 'Bug',
			'project' => 'Datahub',
			'project_id' => 17,
		);
	}

	private function _bugs_get_projects()
	{
		return array(
			'dh' => 'Datahub',
			'pr' => 'Presenter',
			'wr' => 'Writer'
		);
	}

	private function _bugs_get_types()
	{
		return array(
			1 => 'Bug',
			2 => 'Feature',
			3 => 'Task'
		);
	}

	private function _bugs_get_components($project)
	{
		$components = array();
		switch ($project)
		{
			case 'dh':
				$components = array(
					'User Interface', 'Installer', 'Documentation');
				break;
			
			case 'pr':
				$components = array(
					'Application', 'Installer', 'Import/Export');
				break;

			case 'wr':
				$components = array(
					'Documents', 'Settings', 'Printing');
				break;
		}
		
		return $components;
	}
}